@@ -552,8 +552,8 @@ gdk_event_get_state (GdkEvent        *event,
 /**
  * gdk_event_get_coords:
  * @event: a #GdkEvent
- * @x_root: location to put event window x coordinate
- * @y_root: location to put event window y coordinate
+ * @x_win: location to put event window x coordinate
+ * @y_win: location to put event window y coordinate
  * 
  * Extract the event window relative x/y coordinates from an event.
  * 
@@ -603,7 +603,7 @@ gdk_event_get_coords (GdkEvent *event,
   if (x_win)
     *x_win = x;
   if (y_win)
-    *y_win = x;
+    *y_win = y;
 
   return fetched;
 }
@@ -881,7 +881,7 @@ _gdk_event_unqueue (void)
   return event;
 }
 
-void
+static void
 gdk_synthesize_click (GdkEvent *event,
 		      gint	nclicks)
 {
@@ -941,23 +941,23 @@ gdk_synthesize_window_state (GdkWindow     *window,
                              GdkWindowState unset_flags,
                              GdkWindowState set_flags)
 {
-  GdkEventWindowState temp_event;
+  GdkEvent temp_event;
   GdkWindowState old;
   
   g_return_if_fail (window != NULL);
   
-  temp_event.window = window;
-  temp_event.type = GDK_WINDOW_STATE;
-  temp_event.send_event = FALSE;
+  temp_event.window_state.window = window;
+  temp_event.window_state.type = GDK_WINDOW_STATE;
+  temp_event.window_state.send_event = FALSE;
   
-  old = ((GdkWindowObject*) temp_event.window)->state;
+  old = ((GdkWindowObject*) temp_event.window_state.window)->state;
   
-  temp_event.changed_mask = (unset_flags | set_flags) ^ old;
-  temp_event.new_window_state = old;
-  temp_event.new_window_state |= set_flags;
-  temp_event.new_window_state &= ~unset_flags;
+  temp_event.window_state.changed_mask = (unset_flags | set_flags) ^ old;
+  temp_event.window_state.new_window_state = old;
+  temp_event.window_state.new_window_state |= set_flags;
+  temp_event.window_state.new_window_state &= ~unset_flags;
 
-  if (temp_event.new_window_state == old)
+  if (temp_event.window_state.new_window_state == old)
     return; /* No actual work to do, nothing changed. */
 
   /* Actually update the field in GdkWindow, this is sort of an odd
@@ -965,7 +965,7 @@ gdk_synthesize_window_state (GdkWindow     *window,
    * inconsistent state to the user.
    */
   
-  ((GdkWindowObject*) window)->state = temp_event.new_window_state;
+  ((GdkWindowObject*) window)->state = temp_event.window_state.new_window_state;
 
   /* We only really send the event to toplevels, since
    * all the window states don't apply to non-toplevels.
@@ -977,7 +977,7 @@ gdk_synthesize_window_state (GdkWindow     *window,
     case GDK_WINDOW_TOPLEVEL:
     case GDK_WINDOW_DIALOG:
     case GDK_WINDOW_TEMP: /* ? */
-      gdk_event_put ((GdkEvent*) &temp_event);
+      gdk_event_put (&temp_event);
       break;
       
     case GDK_WINDOW_FOREIGN:
