@@ -51,7 +51,6 @@
 #include <gdkconfig.h>
 
 /* some common magic values */
-#define GDK_NONE	     0L
 #define GDK_CURRENT_TIME     0L
 #define GDK_PARENT_RELATIVE  1L
 
@@ -64,7 +63,6 @@ extern "C" {
 
 /* Type definitions for the basic structures.
  */
-typedef struct _GdkKeyInfo            GdkKeyInfo;
 typedef struct _GdkPoint	      GdkPoint;
 typedef struct _GdkRectangle	      GdkRectangle;
 typedef struct _GdkSegment	      GdkSegment;
@@ -76,7 +74,14 @@ typedef struct _GdkSpan	              GdkSpan;
  * on Win32, wchar_t is unsigned short.
  */
 typedef guint32			    GdkWChar;
-typedef gulong     		    GdkAtom;
+
+typedef struct _GdkAtom            *GdkAtom;
+
+#define GDK_ATOM_TO_POINTER(atom) (atom)
+#define GDK_POINTER_TO_ATOM(ptr)  ((GdkAtom)(ptr))
+
+#define _GDK_MAKE_ATOM(val) ((GdkAtom)GUINT_TO_POINTER(val))
+#define GDK_NONE            _GDK_MAKE_ATOM (0)
 
 #ifdef GDK_NATIVE_WINDOW_POINTER
 typedef gpointer GdkNativeWindow;
@@ -164,16 +169,6 @@ typedef void (*GdkInputFunction) (gpointer	    data,
 
 typedef void (*GdkDestroyNotify) (gpointer data);
 
-/* GdkKeyInfo is a description of a the hardware key and state that
- * can be mapped to some keysym.
- */
-struct _GdkKeyInfo
-{
-  guint keycode;
-  gint  group;
-  gint  level;
-};
-
 struct _GdkPoint
 {
   gint x;
