@@ -25,8 +25,6 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include "config.h"
-
 /* Cannot use TrackMouseEvent, as the stupid WM_MOUSELEAVE message
  * doesn't tell us where the mouse has gone. Thus we cannot use it to
  * generate a correct GdkNotifyType. Pity, as using TrackMouseEvent
@@ -41,26 +39,31 @@
 
 #include <stdio.h>
 
+#include "gdk.h"
 #include "gdkprivate-win32.h"
+#include "gdkinput-win32.h"
+#include "gdkkeysyms.h"
 
 #include <objbase.h>
+
+#if defined (__GNUC__) && defined (HAVE_DIMM_H)
+/* The w32api imm.h clashes a bit with the IE5.5 dimm.h */
+# define IMEMENUITEMINFOA hidden_IMEMENUITEMINFOA
+# define IMEMENUITEMINFOW hidden_IMEMENUITEMINFOW
+#endif
+
 #include <imm.h>
 
+#if defined (__GNUC__) && defined (HAVE_DIMM_H)
+# undef IMEMENUITEMINFOA
+# undef IMEMENUITEMINFOW
+#endif
+
 #ifdef HAVE_DIMM_H
 #include <dimm.h>
-#else
-#include "surrogate-dimm.h"
 #endif
 
-#include "gdk.h"
-#include "gdkinternals.h"
-#include "gdkinput-win32.h"
 
-#include "gdkkeysyms.h"
-
-#define PING() printf("%s: %d\n",__FILE__,__LINE__),fflush(stdout)
-
-typedef struct _GdkIOClosure GdkIOClosure;
 typedef struct _GdkEventPrivate GdkEventPrivate;
 
 typedef enum
@@ -71,14 +74,6 @@ typedef enum
   GDK_EVENT_PENDING = 1 << 0
 } GdkEventFlags;
 
-struct _GdkIOClosure
-{
-  GdkInputFunction function;
-  GdkInputCondition condition;
-  GdkDestroyNotify notify;
-  gpointer data;
-};
-
 struct _GdkEventPrivate
 {
   GdkEvent event;
@@ -141,8 +136,10 @@ static UINT msh_mousewheel_msg;
 static gboolean ignore_wm_char = FALSE;
 static gboolean is_altgr_key = FALSE;
 
+#ifdef HAVE_DIMM_H
 static IActiveIMMApp *active_imm_app = NULL;
 static IActiveIMMMessagePumpOwner *active_imm_msgpump_owner = NULL;
+#endif
 
 typedef BOOL (WINAPI *PFN_TrackMouseEvent) (LPTRACKMOUSEEVENT);
 static PFN_TrackMouseEvent track_mouse_event = NULL;
@@ -159,7 +156,9 @@ real_window_procedure (HWND   hwnd,
   GdkEvent *eventp;
   MSG msg;
   DWORD pos;
+#ifdef HAVE_DIMM_H
   LRESULT lres;
+#endif
   gint ret_val;
   gboolean ret_val_flag;
 
@@ -261,28 +260,32 @@ real_window_procedure (HWND   hwnd,
     return ret_val;
   else
     {
+#ifndef HAVE_DIMM_H
+      return DefWindowProc (hwnd, message, wparam, lparam);
+#else
       if (active_imm_app == NULL
 	  || (*active_imm_app->lpVtbl->OnDefWindowProc) (active_imm_app, hwnd, message, wparam, lparam, &lres) == S_FALSE)
 	return DefWindowProc (hwnd, message, wparam, lparam);
       else
 	return lres;
+#endif
     }
 }
 
 LRESULT CALLBACK
-gdk_window_procedure (HWND   hwnd,
-		      UINT   message,
-		      WPARAM wparam,
-		      LPARAM lparam)
+_gdk_win32_window_procedure (HWND   hwnd,
+                             UINT   message,
+                             WPARAM wparam,
+                             LPARAM lparam)
 {
   LRESULT retval;
 
-  GDK_NOTE (MISC, g_print ("gdk_window_procedure: %#lx %s\n",
+  GDK_NOTE (MISC, g_print ("_gdk_win32_window_procedure: %#lx %s\n",
 			     (gulong) hwnd, gdk_win32_message_name (message)));
 
   retval = real_window_procedure (hwnd, message, wparam, lparam);
 
-  GDK_NOTE (MISC, g_print ("gdk_window_procedure: %#lx returns %ld\n",
+  GDK_NOTE (MISC, g_print ("_gdk_win32_window_procedure: %#lx returns %ld\n",
 			     (gulong) hwnd, retval));
 
   return retval;
@@ -292,7 +295,9 @@ void
 _gdk_events_init (void)
 {
   GSource *source;
+#ifdef HAVE_DIMM_H
   HRESULT hres;
+#endif
 #ifdef USE_TRACKMOUSEEVENT
   HMODULE user32, imm32;
   HINSTANCE commctrl32;
@@ -319,6 +324,7 @@ _gdk_events_init (void)
   g_source_set_can_recurse (source, TRUE);
   g_source_attach (source, NULL);
 
+#ifdef HAVE_DIMM_H
   hres = CoCreateInstance (&CLSID_CActiveIMM,
 			   NULL,
 			   CLSCTX_ALL,
@@ -336,6 +342,7 @@ _gdk_events_init (void)
 				 active_imm_msgpump_owner));
       (active_imm_msgpump_owner->lpVtbl->Start) (active_imm_msgpump_owner);
     }
+#endif
 
 #ifdef USE_TRACKMOUSEEVENT
   user32 = GetModuleHandle ("user32.dll");
@@ -833,9 +840,9 @@ build_keypress_event (GdkWindowImplWin32 *impl,
 		      MSG                *msg)
 {
   HIMC himc;
-  gint i, bytecount, ucount, ucleft, len;
-  guchar buf[100], *bp;
-  wchar_t wbuf[100], *wcp;
+  gint i, bytecount, ucount;
+  guchar buf[100];
+  wchar_t wbuf[100];
 
   event->key.type = GDK_KEY_PRESS;
   event->key.time = msg->time;
@@ -882,8 +889,7 @@ build_keypress_event (GdkWindowImplWin32 *impl,
        */
       ucount = MultiByteToWideChar (impl->charset_info.ciACP,
 				    0, buf, bytecount,
-				    wbuf, sizeof (wbuf) / sizeof (wbuf[0]));
-      
+				    wbuf, G_N_ELEMENTS (wbuf));
     }
   if (ucount == 0)
     event->key.keyval = GDK_VoidSymbol;
@@ -904,68 +910,18 @@ build_keypress_event (GdkWindowImplWin32 *impl,
   build_key_event_state (event);
 
   /* Build UTF-8 string */
-  ucleft = ucount;
-  len = 0;
-  wcp = wbuf;
-  while (ucleft-- > 0)
+  if (ucount == 1 && wbuf[0] < 0200)
     {
-      wchar_t c = *wcp++;
-
-      if (c < 0x80)
-	len += 1;
-      else if (c < 0x800)
-	len += 2;
-      else
-	len += 3;
+      event->key.string = g_malloc (2);
+      event->key.string[0] = wbuf[0];
+      event->key.string[1] = '\0';
+      event->key.length = 1;
     }
-
-  event->key.string = g_malloc (len + 1);
-  event->key.length = len;
-  
-  ucleft = ucount;
-  wcp = wbuf;
-  bp = event->key.string;
-  while (ucleft-- > 0)
+  else
     {
-      int first;
-      wchar_t c = *wcp++;
-
-      if (c < 0x80)
-	{
-	  first = 0;
-	  len = 1;
-	}
-      else if (c < 0x800)
-	{
-	  first = 0xc0;
-	  len = 2;
-	}
-      else
-	{
-	  first = 0xe0;
-	  len = 3;
-	}
-
-#if 1      
-      /* Woo-hoo! */
-      switch (len)
-	{
-	case 3: bp[2] = (c & 0x3f) | 0x80; c >>= 6; /* Fall through */
-	case 2: bp[1] = (c & 0x3f) | 0x80; c >>= 6; /* Fall through */
-	case 1: bp[0] = c | first;
-	}
-#else
-      for (i = len - 1; i > 0; --i)
-	{
-	  bp[i] = (c & 0x3f) | 0x80;
-	  c >>= 6;
-	}
-      bp[0] = c | first;
-#endif
-
-      bp += len;
+      event->key.string = _gdk_ucs2_to_utf8 (wbuf, ucount);
+      event->key.length = strlen (event->key.string);
     }
-  *bp = 0;
 }
 
 static void
@@ -1065,6 +1021,7 @@ print_event (GdkEvent *event)
     CASE (GDK_NO_EXPOSE);
     CASE (GDK_SCROLL);
     CASE (GDK_WINDOW_STATE);
+    CASE (GDK_SETTING);
 #undef CASE
     }
   g_print ("%#lx ", (gulong) GDK_WINDOW_HWND (event->any.window));
@@ -1639,55 +1596,7 @@ gdk_event_translate (GdkEvent *event,
   /* to translate coordinates to the internal > 16 bit system */
   _gdk_windowing_window_get_offsets (window, &xoffset, &yoffset);
 
-  if (msg->message == gdk_selection_notify_msg)
-    {
-      GDK_NOTE (EVENTS, g_print ("gdk_selection_notify_msg: %#lx\n",
-				 (gulong) msg->hwnd));
-
-      event->selection.type = GDK_SELECTION_NOTIFY;
-      event->selection.window = window;
-      event->selection.selection = msg->wParam;
-      event->selection.target = msg->lParam;
-      event->selection.property = _gdk_selection_property;
-      event->selection.time = msg->time;
-
-      return_val = !GDK_WINDOW_DESTROYED (window);
-
-      goto done;
-    }
-  else if (msg->message == gdk_selection_request_msg)
-    {
-      GDK_NOTE (EVENTS, g_print ("gdk_selection_request_msg: %#lx\n",
-				 (gulong) msg->hwnd));
-
-      event->selection.type = GDK_SELECTION_REQUEST;
-      event->selection.window = window;
-      event->selection.selection = gdk_clipboard_atom;
-      event->selection.target = GDK_TARGET_STRING;
-      event->selection.property = _gdk_selection_property;
-      event->selection.requestor = (guint32) msg->hwnd;
-      event->selection.time = msg->time;
-
-      return_val = !GDK_WINDOW_DESTROYED (window);
-
-      goto done;
-    }
-  else if (msg->message == gdk_selection_clear_msg)
-    {
-      GDK_NOTE (EVENTS, g_print ("gdk_selection_clear_msg: %#lx\n",
-				 (gulong) msg->hwnd));
-
-      event->selection.type = GDK_SELECTION_CLEAR;
-      event->selection.window = window;
-      event->selection.selection = msg->wParam;
-      event->selection.target = msg->lParam;
-      event->selection.time = msg->time;
-
-      return_val = !GDK_WINDOW_DESTROYED (window);
-
-      goto done;
-    }
-  else if (msg->message == msh_mousewheel_msg)
+  if (msg->message == msh_mousewheel_msg)
     {
       GDK_NOTE (EVENTS, g_print ("MSH_MOUSEWHEEL: %#lx %d\n",
 				 (gulong) msg->hwnd, msg->wParam));
@@ -1753,7 +1662,11 @@ gdk_event_translate (GdkEvent *event,
       while (tmp_list)
 	{
 	  GdkClientFilter *filter = tmp_list->data;
-	  if (filter->type == msg->message)
+	  /* FIXME: under win32 messages are not really atoms
+	   * as the following cast suggest, but the appears to be right
+	   * Haven't found a use case though ...
+	   */
+	  if (filter->type == GDK_POINTER_TO_ATOM (msg->message))
 	    {
 	      GDK_NOTE (EVENTS, g_print ("client filter matched\n"));
 	      event->any.window = window;
@@ -1776,7 +1689,8 @@ gdk_event_translate (GdkEvent *event,
 		  return_val = TRUE;
 		  event->client.type = GDK_CLIENT_EVENT;
 		  event->client.window = window;
-		  event->client.message_type = msg->message;
+		  /* FIXME: check if the cast is correct, see above */
+		  event->client.message_type = GDK_POINTER_TO_ATOM (msg->message);
 		  event->client.data_format = 0;
 		  event->client.data.l[0] = msg->wParam;
 		  event->client.data.l[1] = msg->lParam;
@@ -2546,7 +2460,7 @@ gdk_event_translate (GdkEvent *event,
 	  break;
 	}
 
-      colormap = GDK_DRAWABLE_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl)->colormap;
+      colormap = gdk_drawable_get_colormap (window);
       if (colormap)
 	colormap_private = GDK_COLORMAP_PRIVATE_DATA (colormap);
       hdc = (HDC) msg->wParam;
@@ -2589,7 +2503,7 @@ gdk_event_translate (GdkEvent *event,
 
       if (GDK_WINDOW_OBJECT (window)->bg_pixmap == NULL)
 	{
-	  bg = gdk_colormap_color (GDK_DRAWABLE_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl)->colormap,
+	  bg = _gdk_win32_colormap_color (GDK_DRAWABLE_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl)->colormap,
 				   GDK_WINDOW_OBJECT (window)->bg_color.pixel);
 
 	  GetClipBox (hdc, &rect);
@@ -2922,6 +2836,16 @@ gdk_event_translate (GdkEvent *event,
 	  mmi->ptMaxSize.x = MIN(window_impl->hint_max_width, gdk_screen_width ());
 	  mmi->ptMaxSize.y = MIN(window_impl->hint_max_height, gdk_screen_height ());
 	}
+	else if (window_impl->hint_flags & GDK_HINT_MIN_SIZE)
+	{
+	  /* need to initialize */
+	  mmi->ptMaxSize.x = gdk_screen_width ();
+	  mmi->ptMaxSize.y = gdk_screen_height ();
+	}
+	/* lovely API inconsistence: return FALSE when handled */
+	if (ret_val_flagp)
+	  *ret_val_flagp = !(window_impl->hint_flags &
+	                     (GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE));
       break;
 
     case WM_MOVE:
@@ -3155,10 +3079,13 @@ _gdk_events_queue (void)
     {
       GDK_NOTE (EVENTS, g_print ("PeekMessage: %#lx %s\n",
 				 (gulong) msg.hwnd, gdk_win32_message_name (msg.message)));
-
+#ifndef HAVE_DIMM_H
+      TranslateMessage (&msg);
+#else
       if (active_imm_msgpump_owner == NULL
 	  || (active_imm_msgpump_owner->lpVtbl->OnTranslateMessage) (active_imm_msgpump_owner, &msg) != S_OK)
 	TranslateMessage (&msg);
+#endif
 
 #if 1 /* It was like this all the time */
       DispatchMessage (&msg);
