@@ -42,7 +42,9 @@
  */
 
 #include <windows.h>
+#ifdef HAVE_WINTAB
 #include <wintab.h>
+#endif
 #include "gdkinput-win32.h"
 
 static GdkDeviceAxis gdk_input_core_axes[] = {
@@ -50,33 +52,74 @@ static GdkDeviceAxis gdk_input_core_axes[] = {
   { GDK_AXIS_Y, 0, 0 }
 };
 
-GdkDevice gdk_input_core_info =
-{
-  "Core Pointer",
-  GDK_SOURCE_MOUSE,
-  GDK_MODE_SCREEN,
-  TRUE,
-  
-  2,
-  gdk_input_core_axes,
-
-  0,
-  NULL
-};
-
 /* Global variables  */
 
-GDKVAR GdkDevice *gdk_core_pointer = (GdkDevice *)&gdk_input_core_info;
+GDKVAR GdkDevice *_gdk_core_pointer = NULL;
  
-gint              gdk_input_ignore_core;
+gint              _gdk_input_ignore_core;
 
-GList            *gdk_input_devices;
-GList            *gdk_input_windows;
+GList            *_gdk_input_devices;
+GList            *_gdk_input_windows;
+
+void
+_gdk_init_input_core (void)
+{
+  _gdk_core_pointer = g_object_new (GDK_TYPE_DEVICE, NULL);
+  
+  _gdk_core_pointer->name = "Core Pointer";
+  _gdk_core_pointer->source = GDK_SOURCE_MOUSE;
+  _gdk_core_pointer->mode = GDK_MODE_SCREEN;
+  _gdk_core_pointer->has_cursor = TRUE;
+  _gdk_core_pointer->num_axes = 2;
+  _gdk_core_pointer->axes = gdk_input_core_axes;
+  _gdk_core_pointer->num_keys = 0;
+  _gdk_core_pointer->keys = NULL;
+}
+
+static void
+gdk_device_finalize (GObject *object)
+{
+  g_error ("A GdkDevice object was finalized. This should not happen");
+}
+
+static void
+gdk_device_class_init (GObjectClass *class)
+{
+  class->finalize = gdk_device_finalize;
+}
+
+GType
+gdk_device_get_type (void)
+{
+  static GType object_type = 0;
+
+  if (!object_type)
+    {
+      static const GTypeInfo object_info =
+      {
+        sizeof (GdkDeviceClass),
+        (GBaseInitFunc) NULL,
+        (GBaseFinalizeFunc) NULL,
+        (GClassInitFunc) gdk_device_class_init,
+        NULL,           /* class_finalize */
+        NULL,           /* class_data */
+        sizeof (GdkDevicePrivate),
+        0,              /* n_preallocs */
+        (GInstanceInitFunc) NULL,
+      };
+      
+      object_type = g_type_register_static (G_TYPE_OBJECT,
+                                            "GdkDevice",
+                                            &object_info, 0);
+    }
+  
+  return object_type;
+}
 
 GList *
 gdk_devices_list (void)
 {
-  return gdk_input_devices;
+  return _gdk_input_devices;
 }
 
 void
@@ -130,6 +173,31 @@ gdk_device_set_axis_use (GdkDevice   *device,
     }
 }
 
+gboolean
+gdk_device_get_history  (GdkDevice         *device,
+			 GdkWindow         *window,
+			 guint32            start,
+			 guint32            stop,
+			 GdkTimeCoord    ***events,
+			 gint              *n_events)
+{
+  g_return_val_if_fail (window != NULL, FALSE);
+  g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
+  g_return_val_if_fail (events != NULL, FALSE);
+  g_return_val_if_fail (n_events != NULL, FALSE);
+
+  *n_events = 0;
+  *events = NULL;
+
+  if (GDK_WINDOW_DESTROYED (window))
+    return FALSE;
+    
+  if (GDK_IS_CORE (device))
+    return FALSE;
+  else
+    return _gdk_device_get_history (device, window, start, stop, events, n_events);
+}
+
 GdkTimeCoord ** 
 _gdk_device_allocate_history (GdkDevice *device,
 			      gint       n_events)
@@ -161,7 +229,7 @@ gdk_input_window_find(GdkWindow *window)
 {
   GList *tmp_list;
 
-  for (tmp_list=gdk_input_windows; tmp_list; tmp_list=tmp_list->next)
+  for (tmp_list=_gdk_input_windows; tmp_list; tmp_list=tmp_list->next)
     if (((GdkInputWindow *)(tmp_list->data))->window == window)
       return (GdkInputWindow *)(tmp_list->data);
 
@@ -203,11 +271,11 @@ gdk_input_set_extension_events (GdkWindow *window, gint mask,
       iw->num_obscuring = 0;
       iw->grabbed = FALSE;
 
-      gdk_input_windows = g_list_append(gdk_input_windows,iw);
+      _gdk_input_windows = g_list_append(_gdk_input_windows,iw);
       window_private->extension_events = mask;
 
       /* Add enter window events to the event mask */
-      if (g_list_length (gdk_input_devices) > 1)
+      if (g_list_length (_gdk_input_devices) > 1)
 	gdk_window_set_events (window,
 			       gdk_window_get_events (window) | 
 			       GDK_ENTER_NOTIFY_MASK);
@@ -217,14 +285,14 @@ gdk_input_set_extension_events (GdkWindow *window, gint mask,
       iw = gdk_input_window_find (window);
       if (iw)
 	{
-	  gdk_input_windows = g_list_remove(gdk_input_windows,iw);
+	  _gdk_input_windows = g_list_remove(_gdk_input_windows,iw);
 	  g_free(iw);
 	}
 
       window_private->extension_events = 0;
     }
 
-  for (tmp_list = gdk_input_devices; tmp_list; tmp_list = tmp_list->next)
+  for (tmp_list = _gdk_input_devices; tmp_list; tmp_list = tmp_list->next)
     {
       GdkDevicePrivate *gdkdev = tmp_list->data;
 
@@ -247,17 +315,17 @@ gdk_input_window_destroy (GdkWindow *window)
   input_window = gdk_input_window_find (window);
   g_return_if_fail (input_window != NULL);
 
-  gdk_input_windows = g_list_remove (gdk_input_windows,input_window);
+  _gdk_input_windows = g_list_remove (_gdk_input_windows,input_window);
   g_free(input_window);
 }
 
 void
-gdk_input_exit (void)
+_gdk_input_exit (void)
 {
   GList *tmp_list;
   GdkDevicePrivate *gdkdev;
 
-  for (tmp_list = gdk_input_devices; tmp_list; tmp_list = tmp_list->next)
+  for (tmp_list = _gdk_input_devices; tmp_list; tmp_list = tmp_list->next)
     {
       gdkdev = (GdkDevicePrivate *)(tmp_list->data);
       if (!GDK_IS_CORE (gdkdev))
@@ -272,16 +340,19 @@ gdk_input_exit (void)
 	}
     }
 
-  g_list_free(gdk_input_devices);
+  g_list_free(_gdk_input_devices);
 
-  for (tmp_list = gdk_input_windows; tmp_list; tmp_list = tmp_list->next)
+  for (tmp_list = _gdk_input_windows; tmp_list; tmp_list = tmp_list->next)
     g_free(tmp_list->data);
 
-  g_list_free(gdk_input_windows);
+  g_list_free(_gdk_input_windows);
 }
 
 gboolean
-gdk_device_get_axis (GdkDevice *device, gdouble *axes, GdkAxisUse use, gdouble *value)
+gdk_device_get_axis (GdkDevice  *device,
+		     gdouble    *axes,
+		     GdkAxisUse  use,
+		     gdouble    *value)
 {
   gint i;
   
@@ -324,7 +395,7 @@ gdk_device_set_mode (GdkDevice   *device,
   if (mode == GDK_MODE_WINDOW)
     {
       device->has_cursor = FALSE;
-      for (tmp_list = gdk_input_windows; tmp_list; tmp_list = tmp_list->next)
+      for (tmp_list = _gdk_input_windows; tmp_list; tmp_list = tmp_list->next)
 	{
 	  input_window = (GdkInputWindow *)tmp_list->data;
 	  if (input_window->mode != GDK_EXTENSION_EVENTS_CURSOR)
@@ -337,13 +408,13 @@ gdk_device_set_mode (GdkDevice   *device,
   else if (mode == GDK_MODE_SCREEN)
     {
       device->has_cursor = TRUE;
-      for (tmp_list = gdk_input_windows; tmp_list; tmp_list = tmp_list->next)
+      for (tmp_list = _gdk_input_windows; tmp_list; tmp_list = tmp_list->next)
 	_gdk_input_enable_window (((GdkInputWindow *)tmp_list->data)->window,
 				  gdkdev);
     }
   else  /* mode == GDK_MODE_DISABLED */
     {
-      for (tmp_list = gdk_input_windows; tmp_list; tmp_list = tmp_list->next)
+      for (tmp_list = _gdk_input_windows; tmp_list; tmp_list = tmp_list->next)
 	{
 	  input_window = (GdkInputWindow *)tmp_list->data;
 	  if (old_mode != GDK_MODE_WINDOW ||
