@@ -31,7 +31,6 @@
 #include "gdkevents.h"
 #include "gdkpixmap.h"
 #include "gdkwindow.h"
-#include "gdkinternals.h"
 #include "gdkprivate-win32.h"
 #include "gdkinput-win32.h"
 
@@ -53,7 +52,7 @@ static void gdk_window_impl_win32_finalize   (GObject                 *object);
 static gpointer parent_class = NULL;
 
 GType
-gdk_window_impl_win32_get_type (void)
+_gdk_window_impl_win32_get_type (void)
 {
   static GType object_type = 0;
 
@@ -83,7 +82,7 @@ gdk_window_impl_win32_get_type (void)
 GType
 _gdk_window_impl_get_type (void)
 {
-  return gdk_window_impl_win32_get_type ();
+  return _gdk_window_impl_win32_get_type ();
 }
 
 static void
@@ -141,6 +140,8 @@ gdk_window_impl_win32_finalize (GObject *object)
 
   if (window_impl->hcursor != NULL)
     {
+      if (GetCursor () == window_impl->hcursor)
+	SetCursor (NULL);
       if (!DestroyCursor (window_impl->hcursor))
         WIN32_GDI_FAILED("DestroyCursor");
       window_impl->hcursor = NULL;
@@ -178,15 +179,18 @@ gdk_window_impl_win32_set_colormap (GdkDrawable *drawable,
   GdkDrawableImplWin32 *draw_impl;
   
   g_return_if_fail (GDK_IS_WINDOW_IMPL_WIN32 (drawable));
-  g_return_if_fail (gdk_colormap_get_visual (cmap) != gdk_drawable_get_visual (drawable));
 
   impl = GDK_WINDOW_IMPL_WIN32 (drawable);
   draw_impl = GDK_DRAWABLE_IMPL_WIN32 (drawable);
 
-  GDK_DRAWABLE_GET_CLASS (draw_impl)->set_colormap (drawable, cmap);
+  /* chain up */
+  GDK_DRAWABLE_CLASS (parent_class)->set_colormap (drawable, cmap);
   
-  /* XXX */
-  g_print("gdk_window_impl_win32_set_colormap: XXX\n");
+  if (cmap)
+    {
+      /* XXX */
+      g_print("gdk_window_impl_win32_set_colormap: XXX\n");
+    }
 }
 
 static void
@@ -253,7 +257,7 @@ _gdk_windowing_window_init (void)
 /* The Win API function AdjustWindowRect may return negative values
  * resulting in obscured title bars. This helper function is coreccting it.
  */
-BOOL
+static BOOL
 SafeAdjustWindowRectEx (RECT* lpRect,
 			DWORD dwStyle, 
 			BOOL  bMenu, 
@@ -284,7 +288,7 @@ SafeAdjustWindowRectEx (RECT* lpRect,
  *   is ever needed (e.g Dialog specific), every such window should
  *   get its own class
  */
-ATOM
+static ATOM
 RegisterGdkClass (GdkWindowType wtype)
 {
   static ATOM klassTOPLEVEL = 0;
@@ -299,7 +303,7 @@ RegisterGdkClass (GdkWindowType wtype)
   wcl.style = 0; /* DON'T set CS_<H,V>REDRAW. It causes total redraw
                   * on WM_SIZE and WM_MOVE. Flicker, Performance!
                   */
-  wcl.lpfnWndProc = gdk_window_procedure;
+  wcl.lpfnWndProc = _gdk_win32_window_procedure;
   wcl.cbClsExtra = 0;
   wcl.cbWndExtra = 0;
   wcl.hInstance = gdk_app_hmodule;
@@ -314,15 +318,12 @@ RegisterGdkClass (GdkWindowType wtype)
 	  hAppIcon = ExtractIcon (gdk_app_hmodule, sLoc, 0);
 	  if (0 == hAppIcon)
 	    {
-	      char *gdklibname = g_strdup_printf ("gdk-%s.dll", GDK_VERSION);
-
-	      hAppIcon = ExtractIcon (gdk_app_hmodule, gdklibname, 0);
-	      g_free (gdklibname);
+	      if (0 != GetModuleFileName (gdk_dll_hinstance, sLoc, _MAX_PATH))
+		hAppIcon = ExtractIcon (gdk_dll_hinstance, sLoc, 0);
 	    }
-	  
-	  if (0 == hAppIcon) 
-	    hAppIcon = LoadIcon (NULL, IDI_APPLICATION);
 	}
+      if (0 == hAppIcon) 
+	hAppIcon = LoadIcon (NULL, IDI_APPLICATION);
     }
 
   wcl.lpszMenuName = NULL;
@@ -900,6 +901,9 @@ gdk_window_hide (GdkWindow *window)
 			       (guint) GDK_WINDOW_HWND (window)));
 
       private->state |= GDK_WINDOW_STATE_WITHDRAWN;
+
+      _gdk_window_clear_update_area (window);
+
       if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_TOPLEVEL)
 	ShowOwnedPopups (GDK_WINDOW_HWND (window), FALSE);
 
@@ -1086,6 +1090,9 @@ gdk_window_reparent (GdkWindow *window,
   GdkWindowImplWin32 *impl;
 
   g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  g_return_if_fail (new_parent == NULL || GDK_IS_WINDOW (new_parent));
+  g_return_if_fail (GDK_WINDOW_TYPE (window) != GDK_WINDOW_ROOT);
 
   if (!new_parent)
     new_parent = _gdk_parent_root;
@@ -1127,6 +1134,7 @@ gdk_window_reparent (GdkWindow *window,
     gdk_window_set_static_win_gravity (window, parent_private->guffaw_gravity);
   
   parent_private->children = g_list_prepend (parent_private->children, window);
+  _gdk_window_init_position (GDK_WINDOW (window_private));
 }
 
 void
@@ -1158,7 +1166,8 @@ _gdk_windowing_window_clear_area (GdkWindow *window,
       hdc = GetDC (GDK_WINDOW_HWND (window));
       IntersectClipRect (hdc, x, y, x + width + 1, y + height + 1);
       SendMessage (GDK_WINDOW_HWND (window), WM_ERASEBKGND, (WPARAM) hdc, 0);
-      ReleaseDC (GDK_WINDOW_HWND (window), hdc);
+      if (!ReleaseDC (GDK_WINDOW_HWND (window), hdc))
+	WIN32_GDI_FAILED ("ReleaseDC");
     }
 }
 
@@ -1259,6 +1268,14 @@ gdk_window_set_hints (GdkWindow *window,
 
   if (flags)
     {
+      GdkGeometry geom;
+      gint        geom_mask = 0;
+
+      geom.min_width  = min_width;
+      geom.min_height = min_height;
+      geom.max_width  = max_width;
+      geom.max_height = max_height;
+
       if (flags & GDK_HINT_POS)
 	{
 	  if (!GetWindowPlacement (GDK_WINDOW_HWND (window), &size_hints))
@@ -1306,47 +1323,12 @@ gdk_window_set_hints (GdkWindow *window,
 	}
 
       if (flags & GDK_HINT_MIN_SIZE)
-	{
-	  rect.left = 0;
-	  rect.top = 0;
-	  rect.right = min_width;
-	  rect.bottom = min_height;
-	  dwStyle = GetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE);
-	  dwExStyle = GetWindowLong (GDK_WINDOW_HWND (window), GWL_EXSTYLE);
-	  SafeAdjustWindowRectEx (&rect, dwStyle, FALSE, dwExStyle);
-	  impl->hint_min_width = rect.right - rect.left;
-	  impl->hint_min_height = rect.bottom - rect.top;
-
-	  /* Also check if he current size of the window is in bounds. */
-	  GetClientRect (GDK_WINDOW_HWND (window), &rect);
-	  if (rect.right < min_width && rect.bottom < min_height)
-	    gdk_window_resize (window, min_width, min_height);
-	  else if (rect.right < min_width)
-	    gdk_window_resize (window, min_width, rect.bottom);
-	  else if (rect.bottom < min_height)
-	    gdk_window_resize (window, rect.right, min_height);
-	}
+        geom_mask |= GDK_HINT_MIN_SIZE;
 
       if (flags & GDK_HINT_MAX_SIZE)
-	{
-	  rect.left = 0;
-	  rect.top = 0;
-	  rect.right = max_width;
-	  rect.bottom = max_height;
-	  dwStyle = GetWindowLong (GDK_WINDOW_HWND (window), GWL_STYLE);
-	  dwExStyle = GetWindowLong (GDK_WINDOW_HWND (window), GWL_EXSTYLE);
-	  AdjustWindowRectEx (&rect, dwStyle, FALSE, dwExStyle);
-	  impl->hint_max_width = rect.right - rect.left;
-	  impl->hint_max_height = rect.bottom - rect.top;
-	  /* Again, check if the window is too large currently. */
-	  GetClientRect (GDK_WINDOW_HWND (window), &rect);
-	  if (rect.right > max_width && rect.bottom > max_height)
-	    gdk_window_resize (window, max_width, max_height);
-	  else if (rect.right > max_width)
-	    gdk_window_resize (window, max_width, rect.bottom);
-	  else if (rect.bottom > max_height)
-	    gdk_window_resize (window, rect.right, max_height);
-	}
+        geom_mask |= GDK_HINT_MAX_SIZE;
+
+      gdk_window_set_geometry_hints (window, &geom, geom_mask);
     }
 }
 
@@ -1360,6 +1342,7 @@ gdk_window_set_geometry_hints (GdkWindow      *window,
   RECT rect;
   DWORD dwStyle;
   DWORD dwExStyle;
+  gint new_width = 0, new_height = 0;
   
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
@@ -1389,13 +1372,20 @@ gdk_window_set_geometry_hints (GdkWindow      *window,
 
       /* Also check if he current size of the window is in bounds */
       GetClientRect (GDK_WINDOW_HWND (window), &rect);
+
       if (rect.right < geometry->min_width
 	  && rect.bottom < geometry->min_height)
-	gdk_window_resize (window, geometry->min_width, geometry->min_height);
+	{
+	  new_width = geometry->min_width; new_height = geometry->min_height;
+	}
       else if (rect.right < geometry->min_width)
-	gdk_window_resize (window, geometry->min_width, rect.bottom);
+	{
+	  new_width = geometry->min_width; new_height = rect.bottom;
+	}
       else if (rect.bottom < geometry->min_height)
-	gdk_window_resize (window, rect.right, geometry->min_height);
+	{
+	  new_width = rect.right; new_height = geometry->min_height;
+	}
     }
   
   if (geom_mask & GDK_HINT_MAX_SIZE)
@@ -1418,12 +1408,22 @@ gdk_window_set_geometry_hints (GdkWindow      *window,
       GetClientRect (GDK_WINDOW_HWND (window), &rect);
       if (rect.right > geometry->max_width
 	  && rect.bottom > geometry->max_height)
-	gdk_window_resize (window, geometry->max_width, geometry->max_height);
+	{
+	  new_width = geometry->max_width; new_height = geometry->max_height;
+	}
       else if (rect.right > geometry->max_width)
-	gdk_window_resize (window, geometry->max_width, rect.bottom);
+	{
+	  new_width = geometry->max_width; new_height = rect.bottom;
+	}
       else if (rect.bottom > geometry->max_height)
-	gdk_window_resize (window, rect.right, geometry->max_height);
+	{
+	  new_width = rect.right; new_height = geometry->max_height;
+	}
     }
+
+  /* finally apply new size constraints */
+  if (new_width != 0 && new_height != 0)
+    gdk_window_resize (window, new_width, new_height);
   
   /* I don't know what to do when called with zero base_width and height. */
   if (geom_mask & GDK_HINT_BASE_SIZE
@@ -1595,6 +1595,7 @@ gdk_window_set_back_pixmap (GdkWindow *window,
   if (parent_relative)
     {
       private->bg_pixmap = GDK_PARENT_RELATIVE_BG;
+      GDK_NOTE (MISC, g_print (G_STRLOC ": setting background pixmap to parent_relative\n"));
     }
   else
     {
@@ -1618,7 +1619,6 @@ gdk_window_set_cursor (GdkWindow *window,
   GdkCursorPrivate *cursor_private;
   HCURSOR hcursor;
   HCURSOR hprevcursor;
-  POINT pt;
   
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
@@ -1626,43 +1626,58 @@ gdk_window_set_cursor (GdkWindow *window,
   impl = GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl);
   cursor_private = (GdkCursorPrivate*) cursor;
   
-  if (!GDK_WINDOW_DESTROYED (window))
+  if (GDK_WINDOW_DESTROYED (window))
+    return;
+
+  if (!cursor)
+    hcursor = NULL;
+  else
+    hcursor = cursor_private->hcursor;
+  
+  GDK_NOTE (MISC, g_print ("gdk_window_set_cursor: %#x %#x\n",
+			   (guint) GDK_WINDOW_HWND (window),
+			   (guint) hcursor));
+
+  /* First get the old cursor, if any (we wait to free the old one
+   * since it may be the current cursor set in the Win32 API right
+   * now).
+   */
+  hprevcursor = impl->hcursor;
+
+  if (hcursor == NULL)
+    impl->hcursor = NULL;
+  else
     {
-      if (!cursor)
-	hcursor = NULL;
-      else
-	hcursor = cursor_private->hcursor;
+      /* We must copy the cursor as it is OK to destroy the GdkCursor
+       * while still in use for some window. See for instance
+       * gimp_change_win_cursor() which calls gdk_window_set_cursor
+       * (win, cursor), and immediately afterwards gdk_cursor_destroy
+       * (cursor).
+       */
+      if ((impl->hcursor = CopyCursor (hcursor)) == NULL)
+	WIN32_API_FAILED ("CopyCursor");
+      GDK_NOTE (MISC, g_print ("...CopyCursor (%#x) = %#x\n",
+			       (guint) hcursor, (guint) impl->hcursor));
+    }
 
-      GDK_NOTE (MISC, g_print ("gdk_window_set_cursor: %#x %#x\n",
-			       (guint) GDK_WINDOW_HWND (window),
-			       (guint) hcursor));
-      hprevcursor = impl->hcursor;
-      if (hcursor == NULL)
-	impl->hcursor = NULL;
-      else
-	{
-	  /* We must copy the cursor as it is OK to destroy the GdkCursor
-	   * while still in use for some window. See for instance
-	   * gimp_change_win_cursor() which calls
-	   * gdk_window_set_cursor (win, cursor), and immediately
-	   * afterwards gdk_cursor_destroy (cursor).
-	   */
-	  impl->hcursor = CopyCursor (hcursor);
-	  GDK_NOTE (MISC, g_print ("...CopyCursor (%#x) = %#x\n",
-				   (guint) hcursor, (guint) impl->hcursor));
+   /* Set new cursor in all cases if we're over our window */
+  if (gdk_window_get_pointer(window, NULL, NULL, NULL) == window)
+    SetCursor (impl->hcursor);
 
-	  if (hprevcursor != NULL && GetCursor () == hprevcursor)
-	    SetCursor (impl->hcursor);
+  /* Destroy the previous cursor: Need to make sure it's no longer in
+   * use before we destroy it, in case we're not over our window but
+   * the cursor is still set to our old one.
+   */
+  if (hprevcursor != NULL)
+    {
+      if (GetCursor() == hprevcursor)
+ 	SetCursor (NULL);
 
-	  if (hprevcursor != NULL)
-	    {
-	      GDK_NOTE (MISC, g_print ("...DestroyCursor (%#x)\n",
-				       (guint) hprevcursor));
-	  
-	      if (!DestroyCursor (hprevcursor))
-		WIN32_API_FAILED ("DestroyCursor");
-	    }
-	}
+      GDK_NOTE (MISC, g_print ("...DestroyCursor (%#x)\n",
+			       (guint) hprevcursor));
+      
+      if (!DestroyCursor (hprevcursor))
+	WIN32_API_FAILED ("DestroyCursor");
     }
 }
 
@@ -1830,7 +1845,7 @@ _gdk_windowing_window_get_pointer (GdkWindow       *window,
 				   GdkModifierType *mask)
 {
   GdkWindow *return_val;
-  POINT pointc, point;
+  POINT screen_point, point;
   HWND hwnd, hwndc;
 
   g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
@@ -1839,8 +1854,8 @@ _gdk_windowing_window_get_pointer (GdkWindow       *window,
     window = _gdk_parent_root;
 
   return_val = NULL;
-  GetCursorPos (&pointc);
-  point = pointc;
+  GetCursorPos (&screen_point);
+  point = screen_point;
   ScreenToClient (GDK_WINDOW_HWND (window), &point);
 
   if (x)
@@ -1849,17 +1864,28 @@ _gdk_windowing_window_get_pointer (GdkWindow       *window,
     *y = point.y;
 
   hwnd = WindowFromPoint (point);
-  point = pointc;
-  ScreenToClient (hwnd, &point);
-  
-  do {
-    hwndc = ChildWindowFromPoint (hwnd, point);
-    ClientToScreen (hwnd, &point);
-    ScreenToClient (hwndc, &point);
-  } while (hwndc != hwnd && (hwnd = hwndc, 1));	/* Ouch! */
-
-  return_val = gdk_win32_handle_table_lookup ((GdkNativeWindow) hwnd);
-
+  if (hwnd != NULL)
+    {
+      gboolean done = FALSE;
+      
+      while (!done)
+	{
+	  point = screen_point;
+	  ScreenToClient (hwnd, &point);
+	  hwndc = ChildWindowFromPoint (hwnd, point);
+	  if (hwndc == NULL)
+	    done = TRUE;
+	  else if (hwndc == hwnd)
+	    done = TRUE;
+	  else
+	    hwnd = hwndc;
+	}
+      
+      return_val = gdk_window_lookup (hwnd);
+    }
+  else
+    return_val = NULL;
+      
   if (mask)
     {
       BYTE kbd[256];
@@ -1886,8 +1912,9 @@ _gdk_windowing_window_get_pointer (GdkWindow       *window,
 }
 
 GdkWindow*
-_gdk_windowing_window_at_pointer (gint *win_x,
-				  gint *win_y)
+_gdk_windowing_window_at_pointer (GdkScreen *screen,
+				  gint      *win_x,
+				  gint      *win_y)
 {
   GdkWindow *window;
   POINT point, pointc;
@@ -2424,7 +2451,7 @@ gdk_window_stick (GdkWindow *window)
       /* "stick" means stick to all desktops _and_ do not scroll with the
        * viewport. i.e. glue to the monitor glass in all cases.
        */
-      g_warning ("gdk_window_stick (0x%X) ???", GDK_WINDOW_HWND (window));
+      g_warning ("gdk_window_stick (%#x) ???", (guint) GDK_WINDOW_HWND (window));
     }
   else
     {
@@ -2445,7 +2472,7 @@ gdk_window_unstick (GdkWindow *window)
 
   if (GDK_WINDOW_IS_MAPPED (window))
     {
-      g_warning ("gdk_window_unstick (0x%X) ???", GDK_WINDOW_HWND (window));
+      g_warning ("gdk_window_unstick (%#x) ???", (guint) GDK_WINDOW_HWND (window));
     }
   else
     {
@@ -2453,7 +2480,6 @@ gdk_window_unstick (GdkWindow *window)
       gdk_synthesize_window_state (window,
                                    GDK_WINDOW_STATE_STICKY,
                                    0);
-
     }
 }
 
@@ -2557,8 +2583,8 @@ gdk_window_set_type_hint (GdkWindow        *window,
    * XXX ???
    */
   GDK_NOTE (MISC,
-            g_print ("gdk_window_set_type_hint (0x%0X)\n",
-                     GDK_WINDOW_HWND (window)));
+            g_print ("gdk_window_set_type_hint (%#x)\n",
+                     (guint) GDK_WINDOW_HWND (window)));
 }
 
 void
@@ -2567,8 +2593,6 @@ gdk_window_shape_combine_region (GdkWindow *window,
                                  gint       offset_x,
                                  gint       offset_y)
 {
-  gint xoffset, yoffset;
-  
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
