@@ -408,7 +408,7 @@ make_list (const gchar  *text,
  * @list:     location to store the list of strings or %NULL. The
  *            list should be freed with g_strfreev().
  * 
- * Convert a text property in the giving encoding to
+ * Converts a text property in the giving encoding to
  * a list of UTF-8 strings. 
  * 
  * Return value: the number of strings in the resulting
@@ -528,7 +528,7 @@ gdk_string_to_compound_text (const gchar *str,
  * routines for COMPOUND_TEXT only enforce this in one direction,
  * causing cut-and-paste of \r and \r\n separated text to fail.
  * This routine strips out all non-allowed C0 and C1 characters
- * from the input string and also canonicalizes \r, \r\n, and \n\r to \n
+ * from the input string and also canonicalizes \r, and \r\n to \n
  */
 static gchar * 
 sanitize_utf8 (const gchar *src)
@@ -539,10 +539,10 @@ sanitize_utf8 (const gchar *src)
 
   while (*p)
     {
-      if (*p == '\r' || *p == '\n')
+      if (*p == '\r')
 	{
 	  p++;
-	  if (*p == '\r' || *p == '\n')
+	  if (*p == '\n')
 	    p++;
 
 	  g_string_append_c (result, '\n');
@@ -553,7 +553,7 @@ sanitize_utf8 (const gchar *src)
 	  char buf[7];
 	  gint buflen;
 	  
-	  if (!((ch < 0x20 && ch != '\t') || (ch >= 0x7f && ch < 0xa0)))
+	  if (!((ch < 0x20 && ch != '\t' && ch != '\n') || (ch >= 0x7f && ch < 0xa0)))
 	    {
 	      buflen = g_unichar_to_utf8 (ch, buf);
 	      g_string_append_len (result, buf, buflen);
@@ -570,12 +570,12 @@ sanitize_utf8 (const gchar *src)
  * gdk_utf8_to_string_target:
  * @str: a UTF-8 string
  * 
- * Convert an UTF-8 string into the best possible representation
+ * Converts an UTF-8 string into the best possible representation
  * as a STRING. The representation of characters not in STRING
  * is not specified; it may be as pseudo-escape sequences
  * \x{ABCD}, or it may be in some other form of approximation.
  * 
- * Return value: the newly allocated string, or %NULL if the
+ * Return value: the newly-allocated string, or %NULL if the
  *               conversion failed. (It should not fail for
  *               any properly formed UTF-8 string.)
  **/
@@ -608,10 +608,10 @@ gdk_utf8_to_string_target (const gchar *str)
  * @length:   location to store the length of the data
  *            stored in @ctext
  * 
- * Convert from UTF-8 to compound text. 
+ * Converts from UTF-8 to compound text. 
  * 
  * Return value: %TRUE if the conversion succeeded, otherwise
- *               false.
+ *               %FALSE.
  **/
 gboolean
 gdk_utf8_to_compound_text (const gchar *str,
