@@ -88,7 +88,7 @@ guint	   gtk_accel_label_get_accel_width   (GtkAccelLabel *accel_label);
 void	   gtk_accel_label_set_accel_widget  (GtkAccelLabel *accel_label,
 					      GtkWidget	    *accel_widget);
 void	   gtk_accel_label_set_accel_closure (GtkAccelLabel *accel_label,
-					      GClosure	    *closure);
+					      GClosure	    *accel_closure);
 gboolean   gtk_accel_label_refetch           (GtkAccelLabel *accel_label);
 
 #ifdef __cplusplus
