@@ -83,6 +83,8 @@ gtk_invisible_class_init (GtkInvisibleClass *class)
 static void
 gtk_invisible_init (GtkInvisible *invisible)
 {
+  GdkColormap *colormap;
+  
   GTK_WIDGET_UNSET_FLAGS (invisible, GTK_NO_WINDOW);
   GTK_WIDGET_SET_FLAGS (invisible, GTK_TOPLEVEL);
 
@@ -90,6 +92,10 @@ gtk_invisible_init (GtkInvisible *invisible)
   gtk_object_sink (GTK_OBJECT (invisible));
 
   invisible->has_user_ref_count = TRUE;
+  
+  colormap = _gtk_widget_peek_colormap ();
+  if (colormap)
+    gtk_widget_set_colormap (GTK_WIDGET (invisible), colormap);
 }
 
 static void
@@ -119,7 +125,6 @@ gtk_invisible_realize (GtkWidget *widget)
   GdkWindowAttr attributes;
   gint attributes_mask;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_INVISIBLE (widget));
 
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
