@@ -114,7 +114,16 @@ extern void _gtk_marshal_INT__POINTER (GClosure     *closure,
 #define _gtk_marshal_VOID__INT	g_cclosure_marshal_VOID__INT
 #define _gtk_marshal_NONE__INT	_gtk_marshal_VOID__INT
 
-/* NONE:INT,INT (gtkmarshalers.list:39) */
+/* NONE:INT,BOOL (gtkmarshalers.list:39) */
+extern void _gtk_marshal_VOID__INT_BOOLEAN (GClosure     *closure,
+                                            GValue       *return_value,
+                                            guint         n_param_values,
+                                            const GValue *param_values,
+                                            gpointer      invocation_hint,
+                                            gpointer      marshal_data);
+#define _gtk_marshal_NONE__INT_BOOL	_gtk_marshal_VOID__INT_BOOLEAN
+
+/* NONE:INT,INT (gtkmarshalers.list:40) */
 extern void _gtk_marshal_VOID__INT_INT (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -123,11 +132,11 @@ extern void _gtk_marshal_VOID__INT_INT (GClosure     *closure,
                                         gpointer      marshal_data);
 #define _gtk_marshal_NONE__INT_INT	_gtk_marshal_VOID__INT_INT
 
-/* NONE:NONE (gtkmarshalers.list:40) */
+/* NONE:NONE (gtkmarshalers.list:41) */
 #define _gtk_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID
 #define _gtk_marshal_NONE__NONE	_gtk_marshal_VOID__VOID
 
-/* NONE:STRING,INT,POINTER (gtkmarshalers.list:41) */
+/* NONE:STRING,INT,POINTER (gtkmarshalers.list:42) */
 extern void _gtk_marshal_VOID__STRING_INT_POINTER (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -136,7 +145,7 @@ extern void _gtk_marshal_VOID__STRING_INT_POINTER (GClosure     *closure,
                                                    gpointer      marshal_data);
 #define _gtk_marshal_NONE__STRING_INT_POINTER	_gtk_marshal_VOID__STRING_INT_POINTER
 
-/* STRING:DOUBLE (gtkmarshalers.list:42) */
+/* STRING:DOUBLE (gtkmarshalers.list:43) */
 extern void _gtk_marshal_STRING__DOUBLE (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -144,9 +153,9 @@ extern void _gtk_marshal_STRING__DOUBLE (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:BOOLEAN (gtkmarshalers.list:43) */
+/* VOID:BOOLEAN (gtkmarshalers.list:44) */
 
-/* VOID:BOOLEAN,BOOLEAN,BOOLEAN (gtkmarshalers.list:44) */
+/* VOID:BOOLEAN,BOOLEAN,BOOLEAN (gtkmarshalers.list:45) */
 extern void _gtk_marshal_VOID__BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -154,10 +163,10 @@ extern void _gtk_marshal_VOID__BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
-/* VOID:BOXED (gtkmarshalers.list:45) */
+/* VOID:BOXED (gtkmarshalers.list:46) */
 #define _gtk_marshal_VOID__BOXED	g_cclosure_marshal_VOID__BOXED
 
-/* VOID:BOXED,BOXED (gtkmarshalers.list:46) */
+/* VOID:BOXED,BOXED (gtkmarshalers.list:47) */
 extern void _gtk_marshal_VOID__BOXED_BOXED (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -165,7 +174,7 @@ extern void _gtk_marshal_VOID__BOXED_BOXED (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:BOXED,BOXED,POINTER (gtkmarshalers.list:47) */
+/* VOID:BOXED,BOXED,POINTER (gtkmarshalers.list:48) */
 extern void _gtk_marshal_VOID__BOXED_BOXED_POINTER (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
@@ -173,7 +182,7 @@ extern void _gtk_marshal_VOID__BOXED_BOXED_POINTER (GClosure     *closure,
                                                     gpointer      invocation_hint,
                                                     gpointer      marshal_data);
 
-/* VOID:BOXED,OBJECT (gtkmarshalers.list:48) */
+/* VOID:BOXED,OBJECT (gtkmarshalers.list:49) */
 extern void _gtk_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -181,7 +190,7 @@ extern void _gtk_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:BOXED,STRING,INT (gtkmarshalers.list:49) */
+/* VOID:BOXED,STRING,INT (gtkmarshalers.list:50) */
 extern void _gtk_marshal_VOID__BOXED_STRING_INT (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -189,7 +198,7 @@ extern void _gtk_marshal_VOID__BOXED_STRING_INT (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:BOXED,UINT (gtkmarshalers.list:50) */
+/* VOID:BOXED,UINT (gtkmarshalers.list:51) */
 extern void _gtk_marshal_VOID__BOXED_UINT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -197,7 +206,7 @@ extern void _gtk_marshal_VOID__BOXED_UINT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:BOXED,UINT,FLAGS (gtkmarshalers.list:51) */
+/* VOID:BOXED,UINT,FLAGS (gtkmarshalers.list:52) */
 extern void _gtk_marshal_VOID__BOXED_UINT_FLAGS (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -205,7 +214,7 @@ extern void _gtk_marshal_VOID__BOXED_UINT_FLAGS (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:BOXED,UINT,UINT (gtkmarshalers.list:52) */
+/* VOID:BOXED,UINT,UINT (gtkmarshalers.list:53) */
 extern void _gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -213,9 +222,9 @@ extern void _gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
-/* VOID:ENUM (gtkmarshalers.list:53) */
+/* VOID:ENUM (gtkmarshalers.list:54) */
 
-/* VOID:ENUM,ENUM (gtkmarshalers.list:54) */
+/* VOID:ENUM,ENUM (gtkmarshalers.list:55) */
 extern void _gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -223,7 +232,7 @@ extern void _gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:ENUM,FLOAT (gtkmarshalers.list:55) */
+/* VOID:ENUM,FLOAT (gtkmarshalers.list:56) */
 extern void _gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -231,7 +240,7 @@ extern void _gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:ENUM,FLOAT,BOOLEAN (gtkmarshalers.list:56) */
+/* VOID:ENUM,FLOAT,BOOLEAN (gtkmarshalers.list:57) */
 extern void _gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -239,7 +248,7 @@ extern void _gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:ENUM,INT (gtkmarshalers.list:57) */
+/* VOID:ENUM,INT (gtkmarshalers.list:58) */
 extern void _gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -247,7 +256,7 @@ extern void _gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:ENUM,INT,BOOLEAN (gtkmarshalers.list:58) */
+/* VOID:ENUM,INT,BOOLEAN (gtkmarshalers.list:59) */
 extern void _gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -255,11 +264,11 @@ extern void _gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:INT (gtkmarshalers.list:59) */
+/* VOID:INT (gtkmarshalers.list:60) */
 
-/* VOID:INT,INT (gtkmarshalers.list:60) */
+/* VOID:INT,INT (gtkmarshalers.list:61) */
 
-/* VOID:INT,INT,BOXED (gtkmarshalers.list:61) */
+/* VOID:INT,INT,BOXED (gtkmarshalers.list:62) */
 extern void _gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -267,7 +276,7 @@ extern void _gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:INT,INT,INT (gtkmarshalers.list:62) */
+/* VOID:INT,INT,INT (gtkmarshalers.list:63) */
 extern void _gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -275,10 +284,10 @@ extern void _gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:OBJECT (gtkmarshalers.list:63) */
+/* VOID:OBJECT (gtkmarshalers.list:64) */
 #define _gtk_marshal_VOID__OBJECT	g_cclosure_marshal_VOID__OBJECT
 
-/* VOID:OBJECT,BOOLEAN (gtkmarshalers.list:64) */
+/* VOID:OBJECT,BOOLEAN (gtkmarshalers.list:65) */
 extern void _gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -286,7 +295,7 @@ extern void _gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* VOID:OBJECT,BOXED,BOXED (gtkmarshalers.list:65) */
+/* VOID:OBJECT,BOXED,BOXED (gtkmarshalers.list:66) */
 extern void _gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -294,7 +303,7 @@ extern void _gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshalers.list:66) */
+/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshalers.list:67) */
 extern void _gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                                        GValue       *return_value,
                                                        guint         n_param_values,
@@ -302,7 +311,7 @@ extern void _gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                                        gpointer      invocation_hint,
                                                        gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT (gtkmarshalers.list:67) */
+/* VOID:OBJECT,INT,INT (gtkmarshalers.list:68) */
 extern void _gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -310,7 +319,7 @@ extern void _gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshalers.list:68) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshalers.list:69) */
 extern void _gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                                GValue       *return_value,
                                                                guint         n_param_values,
@@ -318,7 +327,7 @@ extern void _gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *clo
                                                                gpointer      invocation_hint,
                                                                gpointer      marshal_data);
 
-/* VOID:OBJECT,OBJECT (gtkmarshalers.list:69) */
+/* VOID:OBJECT,OBJECT (gtkmarshalers.list:70) */
 extern void _gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -326,7 +335,7 @@ extern void _gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:OBJECT,STRING,STRING (gtkmarshalers.list:70) */
+/* VOID:OBJECT,STRING,STRING (gtkmarshalers.list:71) */
 extern void _gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                                      GValue       *return_value,
                                                      guint         n_param_values,
@@ -334,7 +343,7 @@ extern void _gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                                      gpointer      invocation_hint,
                                                      gpointer      marshal_data);
 
-/* VOID:OBJECT,UINT (gtkmarshalers.list:71) */
+/* VOID:OBJECT,UINT (gtkmarshalers.list:72) */
 extern void _gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -342,7 +351,7 @@ extern void _gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:OBJECT,UINT,FLAGS (gtkmarshalers.list:72) */
+/* VOID:OBJECT,UINT,FLAGS (gtkmarshalers.list:73) */
 extern void _gtk_marshal_VOID__OBJECT_UINT_FLAGS (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -350,10 +359,10 @@ extern void _gtk_marshal_VOID__OBJECT_UINT_FLAGS (GClosure     *closure,
                                                   gpointer      invocation_hint,
                                                   gpointer      marshal_data);
 
-/* VOID:POINTER (gtkmarshalers.list:73) */
+/* VOID:POINTER (gtkmarshalers.list:74) */
 #define _gtk_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER
 
-/* VOID:POINTER,INT (gtkmarshalers.list:74) */
+/* VOID:POINTER,INT (gtkmarshalers.list:75) */
 extern void _gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -361,7 +370,7 @@ extern void _gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER,POINTER (gtkmarshalers.list:75) */
+/* VOID:POINTER,POINTER,POINTER (gtkmarshalers.list:76) */
 extern void _gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -369,7 +378,7 @@ extern void _gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
-/* VOID:POINTER,UINT (gtkmarshalers.list:76) */
+/* VOID:POINTER,UINT (gtkmarshalers.list:77) */
 extern void _gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -377,10 +386,10 @@ extern void _gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:STRING (gtkmarshalers.list:77) */
+/* VOID:STRING (gtkmarshalers.list:78) */
 #define _gtk_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING
 
-/* VOID:STRING,STRING (gtkmarshalers.list:78) */
+/* VOID:STRING,STRING (gtkmarshalers.list:79) */
 extern void _gtk_marshal_VOID__STRING_STRING (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -388,9 +397,9 @@ extern void _gtk_marshal_VOID__STRING_STRING (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:STRING,INT,POINTER (gtkmarshalers.list:79) */
+/* VOID:STRING,INT,POINTER (gtkmarshalers.list:80) */
 
-/* VOID:UINT,UINT (gtkmarshalers.list:80) */
+/* VOID:UINT,UINT (gtkmarshalers.list:81) */
 extern void _gtk_marshal_VOID__UINT_UINT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -398,7 +407,7 @@ extern void _gtk_marshal_VOID__UINT_UINT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:UINT,UINT,BOXED (gtkmarshalers.list:81) */
+/* VOID:UINT,UINT,BOXED (gtkmarshalers.list:82) */
 extern void _gtk_marshal_VOID__UINT_UINT_BOXED (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -406,7 +415,7 @@ extern void _gtk_marshal_VOID__UINT_UINT_BOXED (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
-/* VOID:UINT,STRING (gtkmarshalers.list:82) */
+/* VOID:UINT,STRING (gtkmarshalers.list:83) */
 extern void _gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -414,7 +423,7 @@ extern void _gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshalers.list:83) */
+/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshalers.list:84) */
 extern void _gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
@@ -422,7 +431,7 @@ extern void _gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closur
                                                             gpointer      invocation_hint,
                                                             gpointer      marshal_data);
 
-/* VOID:UINT,OBJECT,UINT,FLAGS,FLAGS (gtkmarshalers.list:84) */
+/* VOID:UINT,OBJECT,UINT,FLAGS,FLAGS (gtkmarshalers.list:85) */
 extern void _gtk_marshal_VOID__UINT_OBJECT_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
@@ -430,7 +439,7 @@ extern void _gtk_marshal_VOID__UINT_OBJECT_UINT_FLAGS_FLAGS (GClosure     *closu
                                                              gpointer      invocation_hint,
                                                              gpointer      marshal_data);
 
-/* VOID:VOID (gtkmarshalers.list:85) */
+/* VOID:VOID (gtkmarshalers.list:86) */
 
 G_END_DECLS
 
