@@ -210,7 +210,7 @@ gtk_plug_set_is_child (GtkPlug  *plug,
  * @plug: a #GtkPlug
  * @socket: a #GtkSocket
  * 
- * Add a plug to a socket within the same application.
+ * Adds a plug to a socket within the same application.
  **/
 void
 _gtk_plug_add_to_socket (GtkPlug   *plug,
@@ -244,7 +244,7 @@ _gtk_plug_add_to_socket (GtkPlug   *plug,
  * @plug: a #GtkPlug
  * @socket: a #GtkSocket
  * 
- * Remove a plug from a socket within the same application.
+ * Removes a plug from a socket within the same application.
  **/
 void
 _gtk_plug_remove_from_socket (GtkPlug   *plug,
@@ -540,6 +540,8 @@ gtk_plug_size_allocate (GtkWidget     *widget,
     {
       GtkBin *bin = GTK_BIN (widget);
 
+      widget->allocation = *allocation;
+
       if (GTK_WIDGET_REALIZED (widget))
 	gdk_window_move_resize (widget->window,
 				allocation->x, allocation->y,
@@ -565,13 +567,18 @@ static gboolean
 gtk_plug_key_press_event (GtkWidget   *widget,
 			  GdkEventKey *event)
 {
-  if (!GTK_WINDOW (widget)->has_focus)
+  if (GTK_WIDGET_TOPLEVEL (widget))
     {
-      gtk_plug_forward_key_press (GTK_PLUG (widget), event);
-      return TRUE;
+      if (!GTK_WINDOW (widget)->has_focus)
+	{
+	  gtk_plug_forward_key_press (GTK_PLUG (widget), event);
+	  return TRUE;
+	}
+      else
+	return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
     }
   else
-    return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
+    return FALSE;
 }
 
 static void
@@ -909,6 +916,7 @@ handle_modality_on (GtkPlug *plug)
   if (!plug->modality_window)
     {
       plug->modality_window = gtk_window_new (GTK_WINDOW_POPUP);
+      gtk_widget_realize (plug->modality_window);
       gtk_window_group_add_window (plug->modality_group, GTK_WINDOW (plug->modality_window));
       gtk_grab_add (plug->modality_window);
     }
