@@ -70,9 +70,6 @@ struct _GtkPreview
 
 struct _GtkPreviewInfo
 {
-  GdkVisual *visual;
-  GdkColormap *cmap;
-
   guchar *lookup;
 
   gdouble gamma;
@@ -125,8 +122,10 @@ void            gtk_preview_set_install_cmap   (gint             install_cmap);
 void            gtk_preview_set_reserved       (gint             nreserved);
 void            gtk_preview_set_dither         (GtkPreview      *preview,
 						GdkRgbDither     dither);
+#ifndef GTK_DISABLE_DEPRECATED
 GdkVisual*      gtk_preview_get_visual         (void);
 GdkColormap*    gtk_preview_get_cmap           (void);
+#endif
 GtkPreviewInfo* gtk_preview_get_info           (void);
 
 /* This function reinitializes the preview colormap and visual from
