@@ -37,6 +37,7 @@
 extern "C" {
 #endif /* __cplusplus */
 
+#ifndef GTK_DISABLE_DEPRECATED
 
 #define GTK_TYPE_PROGRESS            (gtk_progress_get_type ())
 #define GTK_PROGRESS(obj)            (GTK_CHECK_CAST ((obj), GTK_TYPE_PROGRESS, GtkProgress))
@@ -45,6 +46,7 @@ extern "C" {
 #define GTK_IS_PROGRESS_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_PROGRESS))
 #define GTK_PROGRESS_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_PROGRESS, GtkProgressClass))
 
+#endif /* GTK_DISABLE_DEPRECATED */
 
 typedef struct _GtkProgress       GtkProgress;
 typedef struct _GtkProgressClass  GtkProgressClass;
@@ -78,9 +80,11 @@ struct _GtkProgressClass
  * directly.
  */
 
+#ifndef GTK_DISABLE_DEPRECATED
+
 GtkType    gtk_progress_get_type            (void) G_GNUC_CONST;
 void       gtk_progress_set_show_text       (GtkProgress   *progress,
-					     gint           show_text);
+					     gboolean       show_text);
 void       gtk_progress_set_text_alignment  (GtkProgress   *progress,
 					     gfloat         x_align,
 					     gfloat         y_align);
@@ -89,23 +93,24 @@ void       gtk_progress_set_format_string   (GtkProgress   *progress,
 void       gtk_progress_set_adjustment      (GtkProgress   *progress,
 					     GtkAdjustment *adjustment);
 void       gtk_progress_configure           (GtkProgress   *progress,
-					     gfloat         value,
-					     gfloat         min,
-					     gfloat         max);
+					     gdouble        value,
+					     gdouble        min,
+					     gdouble        max);
 void       gtk_progress_set_percentage      (GtkProgress   *progress,
-					     gfloat         percentage);
+					     gdouble        percentage);
 void       gtk_progress_set_value           (GtkProgress   *progress,
-					     gfloat         value);
-gfloat     gtk_progress_get_value           (GtkProgress   *progress);
+					     gdouble        value);
+gdouble    gtk_progress_get_value           (GtkProgress   *progress);
 void       gtk_progress_set_activity_mode   (GtkProgress   *progress,
-					     guint          activity_mode);
+					     gboolean       activity_mode);
 gchar*     gtk_progress_get_current_text    (GtkProgress   *progress);
 gchar*     gtk_progress_get_text_from_value (GtkProgress   *progress,
-					     gfloat         value);
-gfloat     gtk_progress_get_current_percentage (GtkProgress *progress);
-gfloat     gtk_progress_get_percentage_from_value (GtkProgress *progress,
-						   gfloat       value);
+					     gdouble        value);
+gdouble    gtk_progress_get_current_percentage (GtkProgress *progress);
+gdouble    gtk_progress_get_percentage_from_value (GtkProgress *progress,
+						   gdouble      value);
 
+#endif /* GTK_DISABLE_DEPRECATED */
 
 #ifdef __cplusplus
 }
