@@ -452,6 +452,7 @@ static void
 gtk_table_init (GtkTable *table)
 {
   GTK_WIDGET_SET_FLAGS (table, GTK_NO_WINDOW);
+  gtk_widget_set_redraw_on_allocate (GTK_WIDGET (table), FALSE);
   
   table->children = NULL;
   table->rows = NULL;
@@ -639,8 +640,6 @@ gtk_table_set_row_spacing (GtkTable *table,
       if (GTK_WIDGET_VISIBLE (table))
 	gtk_widget_queue_resize (GTK_WIDGET (table));
     }
-
-  g_object_notify (G_OBJECT (table), "row_spacing");
 }
 
 /**
@@ -678,8 +677,6 @@ gtk_table_set_col_spacing (GtkTable *table,
       if (GTK_WIDGET_VISIBLE (table))
 	gtk_widget_queue_resize (GTK_WIDGET (table));
     }
-
-  g_object_notify (G_OBJECT (table), "column_spacing");
 }
 
 /**
@@ -716,6 +713,8 @@ gtk_table_set_row_spacings (GtkTable *table,
   
   if (GTK_WIDGET_VISIBLE (table))
     gtk_widget_queue_resize (GTK_WIDGET (table));
+
+  g_object_notify (G_OBJECT (table), "row_spacing");
 }
 
 /**
@@ -750,6 +749,8 @@ gtk_table_set_col_spacings (GtkTable *table,
   
   if (GTK_WIDGET_VISIBLE (table))
     gtk_widget_queue_resize (GTK_WIDGET (table));
+
+  g_object_notify (G_OBJECT (table), "column_spacing");
 }
 
 /**
@@ -783,6 +784,8 @@ gtk_table_set_homogeneous (GtkTable *table,
       
       if (GTK_WIDGET_VISIBLE (table))
 	gtk_widget_queue_resize (GTK_WIDGET (table));
+
+      g_object_notify (G_OBJECT (table), "homogeneous");
     }
 }
 
