@@ -874,7 +874,6 @@ do_insert_and_scroll (gpointer             callback_data,
   gtk_text_buffer_get_bounds (buffer, &start, &end);
   mark = gtk_text_buffer_create_mark (buffer, NULL, &end, /* right grav */ FALSE);
 
-  gtk_text_iter_backward_char (&end);
   gtk_text_buffer_insert (buffer, &end,
                           "Hello this is multiple lines of text\n"
                           "Line 1\n"  "Line 2\n"
@@ -1161,13 +1160,10 @@ dialog_response_callback (GtkWidget *dialog, gint response_id, gpointer data)
   buffer = g_object_get_data (G_OBJECT (dialog), "buffer");
 
   gtk_text_buffer_get_bounds (buffer, &start, &end);
-
-  /* Remove trailing newline */
-  gtk_text_iter_backward_char (&end);
   
   search_string = gtk_text_iter_get_text (&start, &end);
 
-  printf ("Searching for `%s'\n", search_string);
+  g_print ("Searching for `%s'\n", search_string);
 
   if (response_id == RESPONSE_FORWARD)
     buffer_search_forward (view->buffer, search_string, view);
@@ -1381,6 +1377,71 @@ do_add_children (gpointer callback_data,
                      GTK_TEXT_WINDOW_RIGHT);
 }
 
+static void
+do_add_focus_children (gpointer callback_data,
+                       guint callback_action,
+                       GtkWidget *widget)
+{
+  View *view = view_from_widget (widget);
+  GtkWidget *child;
+  GtkTextChildAnchor *anchor;
+  GtkTextIter iter;
+  GtkTextView *text_view;
+
+  text_view = GTK_TEXT_VIEW (view->text_view);
+  
+  child = gtk_button_new_with_mnemonic ("Button _A in widget->window");
+
+  gtk_text_view_add_child_in_window (text_view,
+                                     child,
+                                     GTK_TEXT_WINDOW_WIDGET,
+                                     200, 200);
+
+  child = gtk_button_new_with_mnemonic ("Button _B in widget->window");
+
+  gtk_text_view_add_child_in_window (text_view,
+                                     child,
+                                     GTK_TEXT_WINDOW_WIDGET,
+                                     350, 300);
+
+  child = gtk_button_new_with_mnemonic ("Button _C in left window");
+
+  gtk_text_view_add_child_in_window (text_view,
+                                     child,
+                                     GTK_TEXT_WINDOW_LEFT,
+                                     0, 0);
+
+  child = gtk_button_new_with_mnemonic ("Button _D in right window");
+  
+  gtk_text_view_add_child_in_window (text_view,
+                                     child,
+                                     GTK_TEXT_WINDOW_RIGHT,
+                                     0, 0);
+
+  gtk_text_buffer_get_start_iter (view->buffer->buffer, &iter);
+  
+  anchor = gtk_text_buffer_create_child_anchor (view->buffer->buffer, &iter);
+
+  child = gtk_button_new_with_mnemonic ("Button _E in buffer");
+  
+  gtk_text_view_add_child_at_anchor (text_view, child, anchor);
+
+  anchor = gtk_text_buffer_create_child_anchor (view->buffer->buffer, &iter);
+
+  child = gtk_button_new_with_mnemonic ("Button _F in buffer");
+  
+  gtk_text_view_add_child_at_anchor (text_view, child, anchor);
+
+  anchor = gtk_text_buffer_create_child_anchor (view->buffer->buffer, &iter);
+
+  child = gtk_button_new_with_mnemonic ("Button _G in buffer");
+  
+  gtk_text_view_add_child_at_anchor (text_view, child, anchor);
+
+  /* show all the buttons */
+  gtk_widget_show_all (view->text_view);
+}
+
 static void
 view_init_menus (View *view)
 {
@@ -1474,6 +1535,7 @@ static GtkItemFactoryEntry menu_items[] =
   { "/Test/_Example",  	 NULL,         do_example,  0, NULL },
   { "/Test/_Insert and scroll", NULL,         do_insert_and_scroll,  0, NULL },
   { "/Test/_Add fixed children", NULL,         do_add_children,  0, NULL },
+  { "/Test/A_dd focusable children", NULL,    do_add_focus_children,  0, NULL },
 };
 
 static gboolean
@@ -2335,7 +2397,8 @@ line_numbers_expose (GtkWidget      *widget,
   
   g_object_unref (G_OBJECT (layout));
 
-  return TRUE;
+  /* don't stop emission, need to draw children */
+  return FALSE;
 }
 
 static View *
